pub struct Singlepass {
pub(crate) enable_nan_canonicalization: bool,
pub(crate) allow_experimental_unaligned_memory_accesses: bool,
pub(crate) debugger: Option<Debugger>,
pub(crate) experimental_artifact: bool,
pub(crate) middlewares: Vec<Arc<dyn ModuleMiddleware>>,
pub(crate) callbacks: Option<SinglepassCallbacks>,
pub num_threads: NonZero<usize>,
}Fields
enable_nan_canonicalization: boolallow_experimental_unaligned_memory_accesses: booldebugger: Option<Debugger>experimental_artifact: boolmiddlewares: Vec<Arc<dyn ModuleMiddleware>>The middleware chain.
callbacks: Option<SinglepassCallbacks>num_threads: NonZero<usize>The number of threads to use for compilation.
Implementations
Sourceimpl Singlepass
impl Singlepass
Sourcepub fn new() -> Self
pub fn new() -> Self
Creates a new configuration object with the default configuration specified.
Sourcepub fn experimental_artifact(&mut self, enable: bool) -> &mut Self
pub fn experimental_artifact(&mut self, enable: bool) -> &mut Self
Enable the experimental artifact format.
pub fn canonicalize_nans(&mut self, enable: bool) -> &mut Self
Sourcepub fn allow_experimental_unaligned_memory_accesses(
&mut self,
enable: bool,
) -> &mut Self
pub fn allow_experimental_unaligned_memory_accesses( &mut self, enable: bool, ) -> &mut Self
Enable run-time handling of potentially unaligned memory accesses. Unaligned memory accesses occur when you try to read N bytes of data starting from an address that is not evenly divisible by N.
This feature is experimental and currently supports only Cranelift scalar types and Singlepass on RISC-V for integral types.
Sourcepub fn callbacks(&mut self, callbacks: Option<SinglepassCallbacks>) -> &mut Self
pub fn callbacks(&mut self, callbacks: Option<SinglepassCallbacks>) -> &mut Self
Callbacks that will triggered in the different compilation phases in Singlepass.
Sourcepub fn num_threads(&mut self, num_threads: NonZero<usize>) -> &mut Self
pub fn num_threads(&mut self, num_threads: NonZero<usize>) -> &mut Self
Set the number of threads to use for compilation.