pub struct MachineRiscv {
assembler: AssemblerRiscv,
used_gprs: FixedBitSet,
used_fprs: FixedBitSet,
trap_table: TrapTable,
instructions_address_map: Vec<InstructionAddressMap>,
src_loc: u32,
unwind_ops: Vec<(usize, UnwindOps<GPR, FPR>)>,
}Expand description
The RISC-V machine state and code emitter.
Fields
assembler: AssemblerRiscvused_gprs: FixedBitSetused_fprs: FixedBitSettrap_table: TrapTableinstructions_address_map: Vec<InstructionAddressMap>Map from byte offset into wasm function to range of native instructions. Ordered by increasing InstructionAddressMap::srcloc.
src_loc: u32The source location for the current operator.
unwind_ops: Vec<(usize, UnwindOps<GPR, FPR>)>Vector of unwind operations with offset.
Implementations
Sourceimpl MachineRiscv
impl MachineRiscv
Sourcepub fn new(target: Option<Target>) -> Result<Self, CompileError>
pub fn new(target: Option<Target>) -> Result<Self, CompileError>
Creates a new RISC-V machine for code generation.
fn used_gprs_contains(&self, r: &GPR) -> bool
fn used_gprs_insert(&mut self, r: GPR)
fn used_gprs_remove(&mut self, r: &GPR) -> bool
fn used_fp_contains(&self, r: &FPR) -> bool
fn used_fprs_insert(&mut self, r: FPR)
fn used_fprs_remove(&mut self, r: &FPR) -> bool
fn location_to_reg( &mut self, sz: Size, src: Location<GPR, FPR>, temps: &mut Vec<GPR>, allow_imm: ImmType, read_val: bool, wanted: Option<GPR>, ) -> Result<Location<GPR, FPR>, CompileError>
fn location_to_fpr( &mut self, sz: Size, src: Location<GPR, FPR>, temps: &mut Vec<FPR>, allow_imm: ImmType, read_val: bool, ) -> Result<Location<GPR, FPR>, CompileError>
fn emit_relaxed_binop( &mut self, op: fn(&mut VecAssembler<RiscvRelocation>, Size, Location<GPR, FPR>, Location<GPR, FPR>) -> Result<(), CompileError>, sz: Size, src: Location<GPR, FPR>, dst: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_relaxed_binop_fp( &mut self, op: fn(&mut VecAssembler<RiscvRelocation>, Size, Location<GPR, FPR>, Location<GPR, FPR>) -> Result<(), CompileError>, sz: Size, src: Location<GPR, FPR>, dst: Location<GPR, FPR>, putback: bool, ) -> Result<(), CompileError>
fn emit_relaxed_binop3( &mut self, op: fn(&mut VecAssembler<RiscvRelocation>, Size, Location<GPR, FPR>, Location<GPR, FPR>, Location<GPR, FPR>) -> Result<(), CompileError>, sz: Size, src1: Location<GPR, FPR>, src2: Location<GPR, FPR>, dst: Location<GPR, FPR>, allow_imm: ImmType, ) -> Result<(), CompileError>
fn emit_relaxed_atomic_binop3( &mut self, op: AtomicBinaryOp, sz: Size, dst: Location<GPR, FPR>, addr: GPR, src: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_relaxed_atomic_cmpxchg( &mut self, size: Size, dst: Location<GPR, FPR>, addr: GPR, new: Location<GPR, FPR>, cmp: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_relaxed_binop3_fp( &mut self, op: fn(&mut VecAssembler<RiscvRelocation>, Size, Location<GPR, FPR>, Location<GPR, FPR>, Location<GPR, FPR>) -> Result<(), CompileError>, sz: Size, src1: Location<GPR, FPR>, src2: Location<GPR, FPR>, dst: Location<GPR, FPR>, allow_imm: ImmType, return_nan_if_present: bool, ) -> Result<(), CompileError>
fn emit_relaxed_cmp( &mut self, c: Condition, loc_a: Location<GPR, FPR>, loc_b: Location<GPR, FPR>, ret: Location<GPR, FPR>, sz: Size, signed: bool, ) -> Result<(), CompileError>
Sourcefn emit_cmpop_i32_dynamic_b(
&mut self,
c: Condition,
loc_a: Location<GPR, FPR>,
loc_b: Location<GPR, FPR>,
ret: Location<GPR, FPR>,
signed: bool,
) -> Result<(), CompileError>
fn emit_cmpop_i32_dynamic_b( &mut self, c: Condition, loc_a: Location<GPR, FPR>, loc_b: Location<GPR, FPR>, ret: Location<GPR, FPR>, signed: bool, ) -> Result<(), CompileError>
I32 comparison with.
Sourcefn emit_cmpop_i64_dynamic_b(
&mut self,
c: Condition,
loc_a: Location<GPR, FPR>,
loc_b: Location<GPR, FPR>,
ret: Location<GPR, FPR>,
) -> Result<(), CompileError>
fn emit_cmpop_i64_dynamic_b( &mut self, c: Condition, loc_a: Location<GPR, FPR>, loc_b: Location<GPR, FPR>, ret: Location<GPR, FPR>, ) -> Result<(), CompileError>
I64 comparison with.
Sourcefn memory_op<F: FnOnce(&mut Self, GPR) -> Result<(), CompileError>>(
&mut self,
addr: Location<GPR, FPR>,
memarg: &MemArg,
check_alignment: bool,
value_size: usize,
imported_memories: bool,
offset: i32,
heap_access_oob: DynamicLabel,
unaligned_atomic: DynamicLabel,
cb: F,
) -> Result<(), CompileError>
fn memory_op<F: FnOnce(&mut Self, GPR) -> Result<(), CompileError>>( &mut self, addr: Location<GPR, FPR>, memarg: &MemArg, check_alignment: bool, value_size: usize, imported_memories: bool, offset: i32, heap_access_oob: DynamicLabel, unaligned_atomic: DynamicLabel, cb: F, ) -> Result<(), CompileError>
NOTE: As observed on the VisionFive 2 board, when an unaligned memory write happens to write out of bounds (and thus triggers SIGSEGV), the memory is partially modified and observable for a subsequent memory read operations. Thus, we always check the boundaries.
fn emit_relaxed_load( &mut self, sz: Size, signed: bool, dst: Location<GPR, FPR>, src: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_maybe_unaligned_load( &mut self, sz: Size, signed: bool, dst: Location<GPR, FPR>, src: GPR, ) -> Result<(), CompileError>
fn emit_relaxed_store( &mut self, sz: Size, dst: Location<GPR, FPR>, src: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_maybe_unaligned_store( &mut self, sz: Size, src: Location<GPR, FPR>, dst: GPR, ) -> Result<(), CompileError>
fn emit_rol( &mut self, sz: Size, loc_a: Location<GPR, FPR>, loc_b: Location<GPR, FPR>, ret: Location<GPR, FPR>, allow_imm: ImmType, ) -> Result<(), CompileError>
fn emit_ror( &mut self, sz: Size, loc_a: Location<GPR, FPR>, loc_b: Location<GPR, FPR>, ret: Location<GPR, FPR>, allow_imm: ImmType, ) -> Result<(), CompileError>
fn emit_popcnt( &mut self, sz: Size, src: Location<GPR, FPR>, dst: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_ctz( &mut self, sz: Size, src: Location<GPR, FPR>, dst: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_clz( &mut self, sz: Size, src: Location<GPR, FPR>, dst: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn convert_float_to_int( &mut self, loc: Location<GPR, FPR>, size_in: Size, ret: Location<GPR, FPR>, size_out: Size, signed: bool, sat: bool, ) -> Result<(), CompileError>
fn convert_int_to_float( &mut self, loc: Location<GPR, FPR>, size_in: Size, ret: Location<GPR, FPR>, size_out: Size, signed: bool, ) -> Result<(), CompileError>
fn convert_float_to_float( &mut self, loc: Location<GPR, FPR>, size_in: Size, ret: Location<GPR, FPR>, size_out: Size, ) -> Result<(), CompileError>
fn trap_float_convertion_errors( &mut self, sz: Size, f: Location<GPR, FPR>, old_fcsr: GPR, temps: &mut Vec<GPR>, ) -> Result<(), CompileError>
fn emit_illegal_op_internal( &mut self, trap: TrapCode, ) -> Result<(), CompileError>
fn emit_relaxed_fcmp( &mut self, c: Condition, size: Size, loc_a: Location<GPR, FPR>, loc_b: Location<GPR, FPR>, ret: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_relaxed_fcvt_with_rounding( &mut self, rounding: RoundingMode, size: Size, loc: Location<GPR, FPR>, ret: Location<GPR, FPR>, ) -> Result<(), CompileError>
fn emit_unwind_op(&mut self, op: UnwindOps<GPR, FPR>)