pub struct MachineX86_64 {
assembler: AssemblerX64,
used_gprs: FixedBitSet,
used_simd: FixedBitSet,
trap_table: TrapTable,
instructions_address_map: Vec<InstructionAddressMap>,
src_loc: u32,
unwind_ops: Vec<(usize, UnwindOps<GPR, XMM>)>,
}Fields
assembler: AssemblerX64used_gprs: FixedBitSetused_simd: FixedBitSettrap_table: TrapTableinstructions_address_map: Vec<InstructionAddressMap>Map from byte offset into wasm function to range of native instructions.
src_loc: u32The source location for the current operator.
unwind_ops: Vec<(usize, UnwindOps<GPR, XMM>)>Vector of unwind operations with offset
Implementations
Sourceimpl MachineX86_64
impl MachineX86_64
pub fn new(target: Option<Target>) -> Result<Self, CompileError>
pub fn emit_relaxed_binop( &mut self, op: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>, sz: Size, src: Location<GPR, XMM>, dst: Location<GPR, XMM>, ) -> Result<(), CompileError>
pub fn emit_relaxed_zx_sx( &mut self, op: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Size, Location<GPR, XMM>) -> Result<(), CompileError>, sz_src: Size, src: Location<GPR, XMM>, sz_dst: Size, dst: Location<GPR, XMM>, ) -> Result<(), CompileError>
Sourcefn emit_binop_i32(
&mut self,
f: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>,
loc_a: Location<GPR, XMM>,
loc_b: Location<GPR, XMM>,
ret: Location<GPR, XMM>,
) -> Result<(), CompileError>
fn emit_binop_i32( &mut self, f: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>, loc_a: Location<GPR, XMM>, loc_b: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
I32 binary operation with both operands popped from the virtual stack.
Sourcefn emit_binop_i64(
&mut self,
f: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>,
loc_a: Location<GPR, XMM>,
loc_b: Location<GPR, XMM>,
ret: Location<GPR, XMM>,
) -> Result<(), CompileError>
fn emit_binop_i64( &mut self, f: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>, loc_a: Location<GPR, XMM>, loc_b: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
I64 binary operation with both operands popped from the virtual stack.
Sourcefn emit_cmpop_i64_dynamic_b(
&mut self,
c: Condition,
loc_a: Location<GPR, XMM>,
loc_b: Location<GPR, XMM>,
ret: Location<GPR, XMM>,
) -> Result<(), CompileError>
fn emit_cmpop_i64_dynamic_b( &mut self, c: Condition, loc_a: Location<GPR, XMM>, loc_b: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
I64 comparison with.
Sourcefn emit_shift_i64(
&mut self,
f: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>,
loc_a: Location<GPR, XMM>,
loc_b: Location<GPR, XMM>,
ret: Location<GPR, XMM>,
) -> Result<(), CompileError>
fn emit_shift_i64( &mut self, f: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>, loc_a: Location<GPR, XMM>, loc_b: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
I64 shift with both operands popped from the virtual stack.
Sourcefn emit_relaxed_xdiv(
&mut self,
op: fn(&mut AssemblerX64, Size, Location<GPR, XMM>) -> Result<(), CompileError>,
sz: Size,
loc: Location<GPR, XMM>,
integer_division_by_zero: DynamicLabel,
) -> Result<usize, CompileError>
fn emit_relaxed_xdiv( &mut self, op: fn(&mut AssemblerX64, Size, Location<GPR, XMM>) -> Result<(), CompileError>, sz: Size, loc: Location<GPR, XMM>, integer_division_by_zero: DynamicLabel, ) -> Result<usize, CompileError>
Moves loc to a valid location for div/idiv.
Sourcefn emit_cmpop_i32_dynamic_b(
&mut self,
c: Condition,
loc_a: Location<GPR, XMM>,
loc_b: Location<GPR, XMM>,
ret: Location<GPR, XMM>,
) -> Result<(), CompileError>
fn emit_cmpop_i32_dynamic_b( &mut self, c: Condition, loc_a: Location<GPR, XMM>, loc_b: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
I32 comparison with.
Sourcefn emit_shift_i32(
&mut self,
f: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>,
loc_a: Location<GPR, XMM>,
loc_b: Location<GPR, XMM>,
ret: Location<GPR, XMM>,
) -> Result<(), CompileError>
fn emit_shift_i32( &mut self, f: fn(&mut AssemblerX64, Size, Location<GPR, XMM>, Location<GPR, XMM>) -> Result<(), CompileError>, loc_a: Location<GPR, XMM>, loc_b: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
I32 shift with both operands popped from the virtual stack.
fn memory_op<F: FnOnce(&mut Self, GPR) -> Result<(), CompileError>>( &mut self, addr: Location<GPR, XMM>, memarg: &MemArg, check_alignment: bool, value_size: usize, need_check: bool, imported_memories: bool, offset: i32, heap_access_oob: DynamicLabel, unaligned_atomic: DynamicLabel, cb: F, ) -> Result<(), CompileError>
fn emit_compare_and_swap<F: FnOnce(&mut Self, GPR, GPR) -> Result<(), CompileError>>( &mut self, loc: Location<GPR, XMM>, target: Location<GPR, XMM>, ret: Location<GPR, XMM>, memarg: &MemArg, value_size: usize, memory_sz: Size, stack_sz: Size, need_check: bool, imported_memories: bool, offset: i32, heap_access_oob: DynamicLabel, unaligned_atomic: DynamicLabel, cb: F, ) -> Result<(), CompileError>
fn emit_f32_int_conv_check( &mut self, reg: XMM, lower_bound: f32, upper_bound: f32, underflow_label: DynamicLabel, overflow_label: DynamicLabel, nan_label: DynamicLabel, succeed_label: DynamicLabel, ) -> Result<(), CompileError>
fn emit_f32_int_conv_check_trap( &mut self, reg: XMM, lower_bound: f32, upper_bound: f32, ) -> Result<(), CompileError>
fn emit_f32_int_conv_check_sat<F1: FnOnce(&mut Self) -> Result<(), CompileError>, F2: FnOnce(&mut Self) -> Result<(), CompileError>, F3: FnOnce(&mut Self) -> Result<(), CompileError>, F4: FnOnce(&mut Self) -> Result<(), CompileError>>( &mut self, reg: XMM, lower_bound: f32, upper_bound: f32, underflow_cb: F1, overflow_cb: F2, nan_cb: Option<F3>, convert_cb: F4, ) -> Result<(), CompileError>
fn emit_f64_int_conv_check( &mut self, reg: XMM, lower_bound: f64, upper_bound: f64, underflow_label: DynamicLabel, overflow_label: DynamicLabel, nan_label: DynamicLabel, succeed_label: DynamicLabel, ) -> Result<(), CompileError>
fn emit_f64_int_conv_check_trap( &mut self, reg: XMM, lower_bound: f64, upper_bound: f64, ) -> Result<(), CompileError>
fn emit_f64_int_conv_check_sat<F1: FnOnce(&mut Self) -> Result<(), CompileError>, F2: FnOnce(&mut Self) -> Result<(), CompileError>, F3: FnOnce(&mut Self) -> Result<(), CompileError>, F4: FnOnce(&mut Self) -> Result<(), CompileError>>( &mut self, reg: XMM, lower_bound: f64, upper_bound: f64, underflow_cb: F1, overflow_cb: F2, nan_cb: Option<F3>, convert_cb: F4, ) -> Result<(), CompileError>
Sourcefn emit_relaxed_avx(
&mut self,
op: fn(&mut AssemblerX64, XMM, XMMOrMemory, XMM) -> Result<(), CompileError>,
src1: Location<GPR, XMM>,
src2: Location<GPR, XMM>,
dst: Location<GPR, XMM>,
) -> Result<(), CompileError>
fn emit_relaxed_avx( &mut self, op: fn(&mut AssemblerX64, XMM, XMMOrMemory, XMM) -> Result<(), CompileError>, src1: Location<GPR, XMM>, src2: Location<GPR, XMM>, dst: Location<GPR, XMM>, ) -> Result<(), CompileError>
Moves src1 and src2 to valid locations and possibly adds a layer of indirection for dst for AVX instructions.
Sourcefn emit_relaxed_avx_base<F: FnOnce(&mut Self, XMM, XMMOrMemory, XMM) -> Result<(), CompileError>>(
&mut self,
op: F,
src1: Location<GPR, XMM>,
src2: Location<GPR, XMM>,
dst: Location<GPR, XMM>,
) -> Result<(), CompileError>
fn emit_relaxed_avx_base<F: FnOnce(&mut Self, XMM, XMMOrMemory, XMM) -> Result<(), CompileError>>( &mut self, op: F, src1: Location<GPR, XMM>, src2: Location<GPR, XMM>, dst: Location<GPR, XMM>, ) -> Result<(), CompileError>
Moves src1 and src2 to valid locations and possibly adds a layer of indirection for dst for AVX instructions.
fn convert_i64_f64_u_s( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i64_f64_u_u( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i64_f64_s_s( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i64_f64_s_u( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i32_f64_s_s( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i32_f64_s_u( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i32_f64_u_s( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i32_f64_u_u( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i64_f32_u_s( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i64_f32_u_u( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i64_f32_s_s( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i64_f32_s_u( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i32_f32_s_s( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i32_f32_s_u( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i32_f32_u_s( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn convert_i32_f32_u_u( &mut self, loc: Location<GPR, XMM>, ret: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn emit_relaxed_atomic_xchg( &mut self, sz: Size, src: Location<GPR, XMM>, dst: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn used_gprs_contains(&self, r: &GPR) -> bool
fn used_simd_contains(&self, r: &XMM) -> bool
fn used_gprs_insert(&mut self, r: GPR)
fn used_simd_insert(&mut self, r: XMM)
fn used_gprs_remove(&mut self, r: &GPR) -> bool
fn used_simd_remove(&mut self, r: &XMM) -> bool
fn emit_unwind_op( &mut self, op: UnwindOps<GPR, XMM>, ) -> Result<(), CompileError>
fn emit_illegal_op_internal( &mut self, trap: TrapCode, ) -> Result<(), CompileError>
fn location_xor( &mut self, size: Size, source: Location<GPR, XMM>, dest: Location<GPR, XMM>, _flags: bool, ) -> Result<(), CompileError>
fn location_or( &mut self, size: Size, source: Location<GPR, XMM>, dest: Location<GPR, XMM>, _flags: bool, ) -> Result<(), CompileError>
fn load_address( &mut self, size: Size, reg: Location<GPR, XMM>, mem: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn location_neg( &mut self, size_val: Size, signed: bool, source: Location<GPR, XMM>, size_op: Size, dest: Location<GPR, XMM>, ) -> Result<(), CompileError>
fn emit_relaxed_zero_extension( &mut self, sz_src: Size, src: Location<GPR, XMM>, sz_dst: Size, dst: Location<GPR, XMM>, ) -> Result<(), CompileError>